@@ -27,7 +27,7 @@
  * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  * 
- * $Id: //depot/r15.2/p4-python/P4API.cpp#2 $
+ * $Id: //depot/r16.2/p4-python/P4API.cpp#1 $
  *
  * Build instructions:
  *  Use Distutils - see accompanying setup.py
@@ -263,7 +263,7 @@ static bool found_error(Error& e)
     return false;
 }
 
-static ServerHelperApi * create_server(char * user, const char * client, const char * directory, ClientUser * ui)
+static ServerHelperApi * create_server(const char * user, const char * client, const char * directory, ClientUser * ui)
 {
     Error e;
 
@@ -281,7 +281,10 @@ static ServerHelperApi * create_server(char * user, const char * client, const c
 
     if( server->Exists(ui, &e)) {
 	StrBuf msg("Personal Server already exists at path '");
-	msg << directory << "'";
+	if( directory )
+	    msg << directory << "'";
+	else
+	    msg << ".'";
 
 	PyErr_SetString(PyExc_RuntimeError, msg.Text());
 	return NULL;
@@ -482,6 +485,18 @@ static PyObject * P4Adapter_parseSpec(P4Adapter * self, PyObject * args)
     return NULL;
 }
 
+static PyObject * P4Adapter_defineSpec(P4Adapter * self, PyObject *args)
+{
+    const char * type;
+    const char * spec;
+
+    if ( PyArg_ParseTuple(args, "ss", &type, &spec) ) {
+	return self->clientAPI->DefineSpec(type, spec);
+    }
+
+    return NULL;
+}
+
 static PyObject * P4Adapter_protocol(P4Adapter * self, PyObject *args)
 {
     const char * var;
@@ -549,6 +564,8 @@ static PyMethodDef P4Adapter_methods[] = {
      "Converts a dictionary-based form into a string"},
     {"parse_spec", (PyCFunction)P4Adapter_parseSpec, METH_VARARGS,
      "Converts a string form into a dictionary"},
+     {"define_spec", (PyCFunction)P4Adapter_defineSpec, METH_VARARGS,
+     "Sets the internal spec for parsing and formating"},
     {"protocol", (PyCFunction)P4Adapter_protocol, METH_VARARGS,
      "Sets a server protocol variable to the given value or gets the protocol level"}, 
     {"disable_tmp_cleanup", (PyCFunction)P4Adapter_disableTmpCleanup, METH_VARARGS,
@@ -986,6 +1003,29 @@ static PyObject *
     return NULL;
 }
 
+static PyObject *
+        P4Map_translateArray(P4Map *self, PyObject * args)
+{
+    // expects two arguments, the string and the direction
+    // True (default) ==> Left to Right
+    // False ==> Right to Left
+
+    PyObject *str;
+    int direction = 1;
+
+#if PY_MAJOR_VERSION >= 3
+    const char * format = "U|b";
+#else
+    const char * format = "S|b";
+#endif
+
+    int ok = PyArg_ParseTuple(args, format, &str, &direction);
+    if (ok) {
+        return self->map->TranslateArray(str, direction);
+    }
+    return NULL;
+}
+
 static PyObject *
         P4Map_reverse(P4Map *self)
 {
@@ -1058,6 +1098,8 @@ static PyMethodDef P4Map_methods[] = {
                 "Clears out the map."},
     {"translate", (PyCFunction) P4Map_translate, METH_VARARGS,
                    "Translates the passed arguments using the map and returns a string"},
+    {"translate_array", (PyCFunction) P4Map_translateArray, METH_VARARGS,
+		                   "Translates the passed arguments using the map and returns an array"},
     {"count", (PyCFunction) P4Map_count, METH_NOARGS,
                    "Returns number of entries in the maps"},
     {"reverse", (PyCFunction) P4Map_reverse, METH_NOARGS,
