@@ -26,6 +26,7 @@ from subprocess import Popen, PIPE
 import sys
 import os, os.path
 import re
+import platform
 
 def onRmTreeError( function, path, exc_info ):
     os.chmod( path, stat.S_IWRITE)
@@ -960,8 +961,16 @@ class TestP4(TestP4Python):
             testAbsoluteDir = os.path.join(self.client_root, testDir)
             os.mkdir(testAbsoluteDir)
 
-            # create a file with windows encoding for its filename
             self.p4.encoding = 'iso8859-1'
+            # create a file with windows encoding for its filename
+            uname = platform.uname()
+            if uname.system == 'Darwin':
+                comp = re.compile('(\d+)\.(\d+)\.(\d+)')
+                match = comp.match(uname.release)
+                major = int(match.group(1))
+                if major >= 16: # macos Sierra or higher
+                    self.p4.encoding = 'utf-8'
+            
             filename = 'öäüÖÄÜß.txt'
             fname = os.path.join(testAbsoluteDir, filename)
             encodedName = fname.encode(self.p4.encoding)
